package at.ac.uibk.gitsearch.config;

/*
 * Copyright 2016-2022 the original author or authors from the JHipster project.
 *
 * This file is part of the JHipster project, see https://www.jhipster.tech/
 * for more information.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.oas.models.info.Contact;
import io.swagger.v3.oas.models.info.Info;
import io.swagger.v3.oas.models.info.License;
import org.springdoc.core.customizers.OpenApiCustomiser;
import org.springframework.core.Ordered;

/**
 * A OpenApi customizer to setup {@link io.swagger.v3.oas.models.OpenAPI} with
 * JHipster settings.
 */
public class SwaggerConfig implements OpenApiCustomiser, Ordered {

    /**
     * The default order for the customizer.
     */
    public static final int DEFAULT_ORDER = 0;

    private int order = DEFAULT_ORDER;

    /** {@inheritDoc} */
    @Override
    public void customise(OpenAPI openAPI) {
        Contact contact = new Contact();
        //
        openAPI.info(
            new Info()
                .contact(contact)
                .title("Sharing Plattform Connector API")
                .description("This is a description of the Sharing Plattform Connector API")
                .version("1.0")
                .termsOfService(null)
                .license(new License().name("License: Open Source, details still to be considered"))
        );
        //        for (JHipsterProperties.ApiDocs.Server server : properties.getServers()) {
        //            openAPI.addServersItem(new Server().url(server.getUrl()).description(server.getDescription()));
        //        }
    }

    /**
     * <p>
     * Setter for the field <code>order</code>.
     * </p>
     *
     * @param order a int.
     */
    public void setOrder(int order) {
        this.order = order;
    }

    /** {@inheritDoc} */
    @Override
    public int getOrder() {
        return order;
    }
}
