package at.ac.uibk.gitsearch.domain;

import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.Cache;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 * A ReviewRating.
 */
@Entity
@Table(name = "review_rating")
@Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
@SuppressWarnings({ "PMD.AvoidFieldNameMatchingMethodName", "PMD.ShortMethodName", "PMD.TooManyMethods" })
public class ReviewRating implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Column(name = "id")
    private Long id;

    @Column(name = "comment")
    private String comment;

    @Enumerated(EnumType.STRING)
    @Column(name = "status")
    private ReviewStatus status;

    @ManyToOne(cascade = { CascadeType.PERSIST, CascadeType.REFRESH, CascadeType.MERGE })
    @JsonIgnoreProperties(value = { "comments", "histories" }, allowSetters = true)
    private Review review;

    @ManyToOne
    private User user;

    @OneToMany(mappedBy = "reviewRating", cascade = CascadeType.ALL)
    @Cache(usage = CacheConcurrencyStrategy.READ_WRITE)
    @JsonIgnoreProperties(value = { "review", "reviewRating" }, allowSetters = true)
    private Set<ReviewHistoryEntry> histories = new HashSet<>();

    // jhipster-needle-entity-add-field - JHipster will add fields here

    public Long getId() {
        return this.id;
    }

    public ReviewRating id(Long id) {
        this.setId(id);
        return this;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getComment() {
        return this.comment;
    }

    public ReviewRating comment(String comment) {
        this.setComment(comment);
        return this;
    }

    public void setComment(String comment) {
        this.comment = comment;
    }

    public ReviewStatus getStatus() {
        return this.status;
    }

    public ReviewRating status(ReviewStatus status) {
        this.setStatus(status);
        return this;
    }

    public void setStatus(ReviewStatus status) {
        this.status = status;
    }

    public Review getReview() {
        return this.review;
    }

    public void setReview(Review review) {
        this.review = review;
    }

    public ReviewRating review(Review review) {
        this.setReview(review);
        return this;
    }

    public User getUser() {
        return this.user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public ReviewRating user(User user) {
        this.setUser(user);
        return this;
    }

    public Set<ReviewHistoryEntry> getHistories() {
        return this.histories;
    }

    public void setHistories(Set<ReviewHistoryEntry> reviewHistoryEntries) {
        if (this.histories != null) {
            this.histories.forEach(i -> i.setReviewRating(null));
        }
        if (reviewHistoryEntries != null) {
            reviewHistoryEntries.forEach(i -> i.setReviewRating(this));
        }
        this.histories = reviewHistoryEntries;
    }

    public ReviewRating histories(Set<ReviewHistoryEntry> reviewHistoryEntries) {
        this.setHistories(reviewHistoryEntries);
        return this;
    }

    public ReviewRating addHistory(ReviewHistoryEntry reviewHistoryEntry) {
        this.histories.add(reviewHistoryEntry);
        reviewHistoryEntry.setReviewRating(this);
        return this;
    }

    public ReviewRating removeHistory(ReviewHistoryEntry reviewHistoryEntry) {
        this.histories.remove(reviewHistoryEntry);
        reviewHistoryEntry.setReviewRating(null);
        return this;
    }

    // jhipster-needle-entity-add-getters-setters - JHipster will add getters and
    // setters here

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof ReviewRating)) {
            return false;
        }
        return id != null && id.equals(((ReviewRating) o).id);
    }

    @Override
    public int hashCode() {
        // see
        // https://vladmihalcea.com/how-to-implement-equals-and-hashcode-using-the-jpa-entity-identifier/
        return getClass().hashCode();
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "ReviewRating{" +
                "id=" + getId() +
                ", comment='" + getComment() + "'" +
                ", status='" + getStatus() + "'" +
                "}";
    }
}
