package at.ac.uibk.gitsearch.repository.jpa;

import at.ac.uibk.gitsearch.domain.ReviewRating;
import at.ac.uibk.gitsearch.domain.User;
import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import java.util.List;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

@Repository
public interface ReviewRatingJPARepository extends JpaRepository<ReviewRating, Long> {
    /**
     * just a nice holder interface for status count
     *
     * @author Michael Breu
     *
     */
    interface StatusAndCount {
        ReviewStatus getStatus();

        Integer getStatusCount();
    }

    @Query("select reviewRating from ReviewRating reviewRating where reviewRating.user.login = ?#{principal.username}")
    List<ReviewRating> findByUserIsCurrentUser();

    void deleteByReviewId(Long reviewId);

    List<ReviewRating> findAllByUserId(Long userId);

    @Query("SELECT rr FROM  ReviewRating rr WHERE rr.review.id=:reviewId")
    List<ReviewRating> findByReview(@Param("reviewId") Long reviewId);

    @Query("SELECT rr.status as status, count(*) as statusCount from ReviewRating rr WHERE rr.user = :user group by status")
    List<StatusAndCount> countReviewsForUserByStatus(@Param("user") User user);

    @Query(
        "SELECT rr.status as status, count(*) as statusCount from Review r LEFT JOIN ReviewRating rr ON rr.review=r.id WHERE r.requestedBy = :userId GROUP BY status"
    )
    List<StatusAndCount> countReviewsForRequestingUserByStatus(@Param("userId") Long userId);

    @Query("SELECT count(*) as statusCount from Review r LEFT JOIN ReviewRating rr ON rr.review=r.id WHERE r.requestedBy = :userId")
    Integer countReviewsRequestedByUser(@Param("userId") Long userId);

    @Query(
        "SELECT count(*) as statusCount from Review r JOIN Statistics s ON r.resourceid = s.exerciseID WHERE r.requestedBy = :userId AND s.badgeRewarded=true "
    )
    Integer countBadgesForRequestingUser(@Param("userId") Long userId);
}
