package at.ac.uibk.gitsearch.service.criteria;

import at.ac.uibk.gitsearch.domain.enumeration.ReviewStatus;
import java.io.Serializable;
import java.util.Objects;
import tech.jhipster.service.Criteria;
import tech.jhipster.service.filter.Filter;
import tech.jhipster.service.filter.LongFilter;
import tech.jhipster.service.filter.StringFilter;

/**
 * Criteria class for the {@link at.ac.uibk.gitsearch.domain.ReviewRating} entity. This class is used
 * in {@link at.ac.uibk.gitsearch.web.rest.ReviewRatingResource} to receive all the possible filtering options from
 * the Http GET request parameters.
 * For example the following could be a valid request:
 * {@code /review-ratings?id.greaterThan=5&attr1.contains=something&attr2.specified=false}
 * As Spring is unable to properly convert the types, unless specific {@link Filter} class are used, we need to use
 * fix type specific filters.
 */
@SuppressWarnings(
    {
        "PMD.GodClass",
        "PMD.AvoidFieldNameMatchingMethodName",
        "PMD.FieldDeclarationsShouldBeAtStartOfClass",
        "PMD.ShortMethodName",
        "PMD.ConfusingTernary",
    }
)
public class ReviewRatingCriteria implements Serializable, Criteria {

    /**
     * Class for filtering ReviewStatus
     */
    public static class ReviewStatusFilter extends Filter<ReviewStatus> {

        public ReviewStatusFilter() {
            super();
        }

        public ReviewStatusFilter(ReviewStatusFilter filter) {
            super(filter);
        }

        @Override
        public ReviewStatusFilter copy() {
            return new ReviewStatusFilter(this);
        }
    }

    private static final long serialVersionUID = 1L;

    private LongFilter id;

    private StringFilter comment;

    private ReviewStatusFilter status;

    private LongFilter reviewId;

    private LongFilter userId;

    private LongFilter historyId;

    private Boolean distinct;

    public ReviewRatingCriteria() {}

    public ReviewRatingCriteria(ReviewRatingCriteria other) {
        this.id = other.id == null ? null : other.id.copy();
        this.comment = other.comment == null ? null : other.comment.copy();
        this.status = other.status == null ? null : other.status.copy();
        this.reviewId = other.reviewId == null ? null : other.reviewId.copy();
        this.userId = other.userId == null ? null : other.userId.copy();
        this.historyId = other.historyId == null ? null : other.historyId.copy();
        this.distinct = other.distinct;
    }

    @Override
    public ReviewRatingCriteria copy() {
        return new ReviewRatingCriteria(this);
    }

    public LongFilter getId() {
        return id;
    }

    public LongFilter id() {
        if (id == null) {
            id = new LongFilter();
        }
        return id;
    }

    public void setId(LongFilter id) {
        this.id = id;
    }

    public StringFilter getComment() {
        return comment;
    }

    public StringFilter comment() {
        if (comment == null) {
            comment = new StringFilter();
        }
        return comment;
    }

    public void setComment(StringFilter comment) {
        this.comment = comment;
    }

    public ReviewStatusFilter getStatus() {
        return status;
    }

    public ReviewStatusFilter status() {
        if (status == null) {
            status = new ReviewStatusFilter();
        }
        return status;
    }

    public void setStatus(ReviewStatusFilter status) {
        this.status = status;
    }

    public LongFilter getReviewId() {
        return reviewId;
    }

    public LongFilter reviewId() {
        if (reviewId == null) {
            reviewId = new LongFilter();
        }
        return reviewId;
    }

    public void setReviewId(LongFilter reviewId) {
        this.reviewId = reviewId;
    }

    public LongFilter getUserId() {
        return userId;
    }

    public LongFilter userId() {
        if (userId == null) {
            userId = new LongFilter();
        }
        return userId;
    }

    public void setUserId(LongFilter userId) {
        this.userId = userId;
    }

    public LongFilter getHistoryId() {
        return historyId;
    }

    public LongFilter historyId() {
        if (historyId == null) {
            historyId = new LongFilter();
        }
        return historyId;
    }

    public void setHistoryId(LongFilter historyId) {
        this.historyId = historyId;
    }

    public Boolean getDistinct() {
        return distinct;
    }

    public void setDistinct(Boolean distinct) {
        this.distinct = distinct;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || getClass() != o.getClass()) {
            return false;
        }
        final ReviewRatingCriteria that = (ReviewRatingCriteria) o;
        return (
            Objects.equals(id, that.id) &&
            Objects.equals(comment, that.comment) &&
            Objects.equals(status, that.status) &&
            Objects.equals(reviewId, that.reviewId) &&
            Objects.equals(userId, that.userId) &&
            Objects.equals(historyId, that.historyId) &&
            Objects.equals(distinct, that.distinct)
        );
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, comment, status, reviewId, userId, historyId, distinct);
    }

    // prettier-ignore
    @Override
    public String toString() {
        return "ReviewRatingCriteria{" +
            (id != null ? "id=" + id + ", " : "") +
            (comment != null ? "comment=" + comment + ", " : "") +
            (status != null ? "status=" + status + ", " : "") +
            (reviewId != null ? "reviewId=" + reviewId + ", " : "") +
            (userId != null ? "userId=" + userId + ", " : "") +
            (historyId != null ? "historyId=" + historyId + ", " : "") +
            (distinct != null ? "distinct=" + distinct + ", " : "") +
            "}";
    }
}
