import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, ActivatedRoute, Router, convertToParamMap } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { of } from 'rxjs';

import { IReviewHistoryEntry, ReviewHistoryEntry } from '../review-history-entry.model';
import { ReviewHistoryEntryService } from '../service/review-history-entry.service';

import { ReviewHistoryEntryRoutingResolveService } from './review-history-entry-routing-resolve.service';

describe('ReviewHistoryEntry routing resolve service', () => {
  let mockRouter: Router;
  let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
  let routingResolveService: ReviewHistoryEntryRoutingResolveService;
  let service: ReviewHistoryEntryService;
  let resultReviewHistoryEntry: IReviewHistoryEntry | undefined;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes([])],
      providers: [
        {
          provide: ActivatedRoute,
          useValue: {
            snapshot: {
              paramMap: convertToParamMap({}),
            },
          },
        },
      ],
    });
    mockRouter = TestBed.inject(Router);
    jest.spyOn(mockRouter, 'navigate').mockImplementation(() => Promise.resolve(true));
    mockActivatedRouteSnapshot = TestBed.inject(ActivatedRoute).snapshot;
    routingResolveService = TestBed.inject(ReviewHistoryEntryRoutingResolveService);
    service = TestBed.inject(ReviewHistoryEntryService);
    resultReviewHistoryEntry = undefined;
  });

  describe('resolve', () => {
    it('should return IReviewHistoryEntry returned by find', () => {
      // GIVEN
      service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultReviewHistoryEntry = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultReviewHistoryEntry).toEqual({ id: 123 });
    });

    it('should return new IReviewHistoryEntry if id is not provided', () => {
      // GIVEN
      service.find = jest.fn();
      mockActivatedRouteSnapshot.params = {};

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultReviewHistoryEntry = result;
      });

      // THEN
      expect(service.find).not.toBeCalled();
      expect(resultReviewHistoryEntry).toEqual(new ReviewHistoryEntry());
    });

    it('should route to 404 page if data not found in server', () => {
      // GIVEN
      jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as ReviewHistoryEntry })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultReviewHistoryEntry = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultReviewHistoryEntry).toEqual(undefined);
      expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
    });
  });
});
