import { Component, OnInit } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { FormBuilder } from '@angular/forms';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs';
import { finalize, map } from 'rxjs/operators';

import { IReviewRating, ReviewRating } from '../review-rating.model';
import { ReviewRatingService } from '../service/review-rating.service';
import { IReview } from 'app/entities/review/review.model';
import { ReviewService } from 'app/entities/review/service/review.service';
import { IUser } from 'app/entities/user/user.model';
import { UserService } from 'app/entities/user/user.service';
import { ReviewStatus } from 'app/entities/enumerations/review-status.model';

@Component({
  selector: 'jhi-review-rating-update',
  templateUrl: './review-rating-update.component.html',
})
export class ReviewRatingUpdateComponent implements OnInit {
  isSaving = false;
  reviewStatusValues = Object.keys(ReviewStatus);

  reviewsSharedCollection: IReview[] = [];
  usersSharedCollection: IUser[] = [];

  editForm = this.fb.group({
    id: [],
    comment: [],
    status: [],
    review: [],
    user: [],
  });

  constructor(
    protected reviewRatingService: ReviewRatingService,
    protected reviewService: ReviewService,
    protected userService: UserService,
    protected activatedRoute: ActivatedRoute,
    protected fb: FormBuilder
  ) {}

  ngOnInit(): void {
    this.activatedRoute.data.subscribe(({ reviewRating }) => {
      this.updateForm(reviewRating);

      this.loadRelationshipsOptions();
    });
  }

  previousState(): void {
    window.history.back();
  }

  save(): void {
    this.isSaving = true;
    const reviewRating = this.createFromForm();
    if (reviewRating.id !== undefined) {
      this.subscribeToSaveResponse(this.reviewRatingService.update(reviewRating));
    } else {
      this.subscribeToSaveResponse(this.reviewRatingService.create(reviewRating));
    }
  }

  trackReviewById(index: number, item: IReview): number {
    return item.id!;
  }

  trackUserById(index: number, item: IUser): number {
    return item.id!;
  }

  protected subscribeToSaveResponse(result: Observable<HttpResponse<IReviewRating>>): void {
    result.pipe(finalize(() => this.onSaveFinalize())).subscribe({
      next: () => this.onSaveSuccess(),
      error: () => this.onSaveError(),
    });
  }

  protected onSaveSuccess(): void {
    this.previousState();
  }

  protected onSaveError(): void {
    // Api for inheritance.
  }

  protected onSaveFinalize(): void {
    this.isSaving = false;
  }

  protected updateForm(reviewRating: IReviewRating): void {
    this.editForm.patchValue({
      id: reviewRating.id,
      comment: reviewRating.comment,
      status: reviewRating.status,
      review: reviewRating.review,
      user: reviewRating.user,
    });

    this.reviewsSharedCollection = this.reviewService.addReviewToCollectionIfMissing(this.reviewsSharedCollection, reviewRating.review);
    this.usersSharedCollection = this.userService.addUserToCollectionIfMissing(this.usersSharedCollection, reviewRating.user);
  }

  protected loadRelationshipsOptions(): void {
    this.reviewService
      .query()
      .pipe(map((res: HttpResponse<IReview[]>) => res.body ?? []))
      .pipe(map((reviews: IReview[]) => this.reviewService.addReviewToCollectionIfMissing(reviews, this.editForm.get('review')!.value)))
      .subscribe((reviews: IReview[]) => (this.reviewsSharedCollection = reviews));

    this.userService
      .query()
      .pipe(map((res: HttpResponse<IUser[]>) => res.body ?? []))
      .pipe(map((users: IUser[]) => this.userService.addUserToCollectionIfMissing(users, this.editForm.get('user')!.value)))
      .subscribe((users: IUser[]) => (this.usersSharedCollection = users));
  }

  protected createFromForm(): IReviewRating {
    return {
      ...new ReviewRating(),
      id: this.editForm.get(['id'])!.value,
      comment: this.editForm.get(['comment'])!.value,
      status: this.editForm.get(['status'])!.value,
      review: this.editForm.get(['review'])!.value,
      user: this.editForm.get(['user'])!.value,
    };
  }
}
