import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { IUserWatchList, UserWatchList } from '../user-watch-list.model';
import { UserWatchListService } from '../service/user-watch-list.service';

@Injectable({ providedIn: 'root' })
export class UserWatchListRoutingResolveService implements Resolve<IUserWatchList> {
  constructor(protected service: UserWatchListService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IUserWatchList> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((userWatchList: HttpResponse<UserWatchList>) => {
          if (userWatchList.body) {
            return of(userWatchList.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new UserWatchList());
  }
}
